/* __placeholder__ */
import { computed, reactive, ref } from 'vue';
import { byteSize } from '../utils/byte-size.utils';
import { useFormWidget } from './FormWidget';
import { set } from '@codeperate/utils';
const { defineProps, defineSlots, defineEmits, defineExpose, defineModel, defineOptions, withDefaults, } = await import('vue');
let __VLS_typeProps;
const props = withDefaults(defineProps(), {
    empty: 'N/A',
    fileSizeBytesLimit: 5000000,
    fileNumberLimit: 5,
    uploadText: 'Drag or click to upload',
    uploadButton: 'Upload',
    fileSizeBytesLimitText: limit => `Maximum upload size per file is ${byteSize(limit)}.`,
    fileSizeBytesLimitErr: 'Upload Failed - Size Exceeded',
    fileNumberLimitText: limit => `Maximum file upload limit is ${limit}.`,
    fileNumberLimitErr: 'Upload Failed - Limit Exceeded',
    mimeTypeLimitErr: 'Invalid file type',
    mimeTypeLimitText: limit => `Allowed file types are ${limit.join(', ')}`,
    uploadFn: async (_, { progress }) => {
        return await new Promise(res => setTimeout(() => {
            progress(100);
            res();
        }, 1000));
    },
    autoUpload: false,
});
const { value, register, setValue, isRequired, form, validatedMeta, validate, validator: defaultValidator, isValid, getPath, } = useFormWidget('FileWidget', props);
const uploading = ref(false);
const dragging = ref(false);
const el = ref();
const fileReader = new FileReader();
const map = reactive(new WeakMap());
const errorMsg = ref('');
const computedErrorMsg = computed(() => errorMsg.value || validatedMeta.value.err?.[0]?.msg);
const processFile = async (files) => {
    let fileInfo = [];
    let url;
    for (let i = 0; i < files.length; i++) {
        if (files[i].type.startsWith('image')) {
            fileReader.readAsDataURL(files[i]);
            url = await new Promise(res => {
                fileReader.addEventListener('load', e => res(e.target.result), { once: true });
            });
        }
        fileInfo.push(files[i]);
        let fileState = {
            progress: 0,
        };
        if (files[i].size > props.fileSizeBytesLimit)
            fileState.error = props.fileSizeBytesLimitErr;
        if (props.mimeTypeLimit && !props.mimeTypeLimit.includes(files[i].type))
            fileState.error = props.mimeTypeLimitErr;
        if (url)
            fileState.image = url;
        map.set(files[i], fileState);
    }
    return fileInfo;
};
const changeHandler = async (e) => {
    if (!(e.target instanceof HTMLInputElement))
        return;
    if (!e.target.files)
        return;
    if ((e.target.files.length ?? 0) + (value.value?.length ?? 0) > props.fileNumberLimit) {
        errorMsg.value = props.fileNumberLimitErr;
        return;
    }
    const fileArr = await processFile(e.target.files);
    if (fileArr.length > 0)
        setValue(fileArr);
    errorMsg.value = '';
    validate();
};
const dropHandler = async (e) => {
    const files = e.dataTransfer?.files;
    if (!files)
        return;
    if ((files.length ?? 0) + (value.value?.length ?? 0) > props.fileNumberLimit) {
        errorMsg.value = props.fileNumberLimitErr;
        return;
    }
    const fileArr = await processFile(files);
    setValue([...(value.value ?? []), ...fileArr]);
    dragging.value = false;
    errorMsg.value = '';
    validate();
};
const deleteHandler = file => {
    setValue(value.value.filter(_file => _file !== file));
    el.value.value = '';
};
const upload = async () => {
    uploading.value = true;
    try {
        for (const file of value.value ?? []) {
            const state = map.get(file);
            if (state?.error)
                continue;
            await props.uploadFn(file, {
                progress: percentage => {
                    state.progress = percentage;
                },
                form,
            });
        }
    }
    catch (err) {
        uploading.value = false;
        throw err;
    }
    uploading.value = false;
};
const __VLS_exposed = { upload };
defineExpose({ upload });
register({
    upload,
    validator: () => {
        defaultValidator();
        if (isRequired() && (value.value?.length ?? 0) <= 0) {
            validatedMeta.value.validity = false;
            el.value?.checkValidity();
            validatedMeta.value.err ??= [{ msg: el.value.validationMessage }];
        }
        return validatedMeta.value;
    },
    el,
    onExportValue: async (exportedValue) => {
        set(exportedValue, [
            'value',
            ...getPath()
                .split('.')
                .filter(s => s),
        ], value.value);
        if (props.uploadOnExport)
            await upload();
    },
});
const __VLS_withDefaultsArg = (function (t) { return t; })({
    empty: 'N/A',
    fileSizeBytesLimit: 5000000,
    fileNumberLimit: 5,
    uploadText: 'Drag or click to upload',
    uploadButton: 'Upload',
    fileSizeBytesLimitText: limit => `Maximum upload size per file is ${byteSize(limit)}.`,
    fileSizeBytesLimitErr: 'Upload Failed - Size Exceeded',
    fileNumberLimitText: limit => `Maximum file upload limit is ${limit}.`,
    fileNumberLimitErr: 'Upload Failed - Limit Exceeded',
    mimeTypeLimitErr: 'Invalid file type',
    mimeTypeLimitText: limit => `Allowed file types are ${limit.join(', ')}`,
    uploadFn: async (_, { progress }) => {
        return await new Promise(res => setTimeout(() => {
            progress(100);
            res();
        }, 1000));
    },
    autoUpload: false,
});
const __VLS_fnComponent = (await import('vue')).defineComponent({});
let __VLS_functionalComponentProps;
function __VLS_template() {
    let __VLS_ctx;
    /* Components */
    let __VLS_otherComponents;
    let __VLS_own;
    let __VLS_localComponents;
    let __VLS_components;
    let __VLS_styleScopedClasses;
    let __VLS_resolvedLocalAndGlobalComponents;
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-grid") }, });
    __VLS_elementAsFunction(__VLS_intrinsicElements.input)({ ...{ onChange: (__VLS_ctx.changeHandler) }, ref: ("el"), ...{ class: ("cfb-hidden") }, type: ("file"), multiple: ((true)), required: ((__VLS_ctx.isRequired())), });
    // @ts-ignore
    (__VLS_ctx.el);
    // @ts-ignore
    [changeHandler, isRequired, el,];
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ onDrop: (__VLS_ctx.dropHandler) }, ...{ onClick: (...[$event]) => {
                if (!__VLS_ctx.uploading)
                    __VLS_ctx.el.click();
                ;
                // @ts-ignore
                [el, dropHandler, uploading,];
            } }, ...{ onDragover: (...[$event]) => {
                __VLS_ctx.dragging = true;
                // @ts-ignore
                [dragging,];
            } }, ...{ onDragenter: (...[$event]) => {
                __VLS_ctx.dragging = true;
                // @ts-ignore
                [dragging,];
            } }, ...{ onDragleave: (...[$event]) => {
                __VLS_ctx.dragging = false;
                // @ts-ignore
                [dragging,];
            } }, ...{ onDragend: (...[$event]) => {
                __VLS_ctx.dragging = false;
                // @ts-ignore
                [dragging,];
            } }, ...{ class: ("cfb-px-2 cfb-py-1") }, });
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: (([
                'cfb-min-h-[18rem] cfb-flex cfb-rounded-lg cfb-ring-2',
                { 'cfb-bg-main-100': __VLS_ctx.dragging },
                __VLS_ctx.validatedMeta.validity ? 'cfb-ring-main-400' : 'cfb-ring-danger-600',
            ])) }, });
    __VLS_styleScopedClasses = ([
        'cfb-min-h-[18rem] cfb-flex cfb-rounded-lg cfb-ring-2',
        { 'cfb-bg-main-100': dragging },
        validatedMeta.validity ? 'cfb-ring-main-400' : 'cfb-ring-danger-600',
    ]);
    if (__VLS_ctx.value?.length) {
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-p-4 cfb-w-full cfb-grid cfb-grid-cols-1 tab:cfb-grid-cols-[repeat(auto-fill,20rem)] cfb-gap-4 cfb-place-self-start") }, });
        for (const [file] of __VLS_getVForSourceType((__VLS_ctx.value))) {
            __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-flex cfb-gap-4 cfb-rounded-lg cfb-p-1 cfb-w-full cfb-cursor-pointer") }, });
            if (__VLS_ctx.map.get(file)?.image) {
                __VLS_elementAsFunction(__VLS_intrinsicElements.img)({ ...{ class: ("cfb-w-20 cfb-rounded-lg cfb-aspect-square cfb-object-cover cfb-self-start") }, src: ((__VLS_ctx.map.get(file)?.image)), });
                // @ts-ignore
                [dragging, validatedMeta, value, value, map, map,];
            }
            else {
                __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-w-20 cfb-shrink-0 cfb-rounded-lg cfb-aspect-square cfb-self-start cfb-bg-gray-100 cfb-flex cfb-justify-center cfb-items-center") }, });
                __VLS_elementAsFunction(__VLS_intrinsicElements.i, __VLS_intrinsicElements.i)({ ...{ class: ("fa-solid fa-file cfb-text-[3rem] cfb-text-main-500") }, });
            }
            __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-text-xs cfb-flex-shrink cfb-min-w-0 cfb-flex-col cfb-flex cfb-flex-grow") }, });
            __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-text-sm cfb-break-words cfb-flex-grow cfb-mb-2") }, });
            (file.name);
            if (__VLS_ctx.map.get(file)?.error) {
                __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-text-danger-600") }, });
                (__VLS_ctx.map.get(file)?.error);
                // @ts-ignore
                [map, map,];
            }
            else {
                __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-self-end") }, });
                (__VLS_ctx.byteSize(file.size));
                // @ts-ignore
                [byteSize,];
                __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-bg-gray-200 cfb-rounded-lg cfb-h-[4px] cfb-relative") }, });
                __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-bg-main-500 cfb-absolute cfb-h-full") }, ...{ style: (({ width: __VLS_ctx.map.get(file)?.progress + '%' })) }, });
                // @ts-ignore
                [map,];
            }
            __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("") }, });
            __VLS_elementAsFunction(__VLS_intrinsicElements.button, __VLS_intrinsicElements.button)({ ...{ onClick: (...[$event]) => {
                        if (!((__VLS_ctx.value?.length)))
                            return;
                        __VLS_ctx.deleteHandler(file);
                        // @ts-ignore
                        [deleteHandler,];
                    } }, disabled: ((__VLS_ctx.uploading)), ...{ class: ("cfb-text-danger-600 hover:cfb-bg-gray-100 cfb-rounded-lg cfb-w-5 cfb-aspect-square disabled:cfb-text-gray-700") }, });
            __VLS_elementAsFunction(__VLS_intrinsicElements.i, __VLS_intrinsicElements.i)({ ...{ class: ("fa-solid fa-ban") }, });
            // @ts-ignore
            [uploading,];
        }
    }
    else {
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-w-full cfb-justify-center cfb-grid cfb-items-center hover:cfb-bg-main-100 cfb-cursor-pointer cfb-rounded-lg") }, });
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: (([
                    'cfb-grid cfb-font-bold cfb-text-lg cfb-text-center cfb-gap-4',
                    { 'cfb-text-danger-600': __VLS_ctx.isValid() == false },
                    { 'cfb-text-main-500': __VLS_ctx.isValid() !== false },
                ])) }, });
        __VLS_styleScopedClasses = ([
            'cfb-grid cfb-font-bold cfb-text-lg cfb-text-center cfb-gap-4',
            { 'cfb-text-danger-600': isValid() == false },
            { 'cfb-text-main-500': isValid() !== false },
        ]);
        __VLS_elementAsFunction(__VLS_intrinsicElements.i, __VLS_intrinsicElements.i)({ ...{ class: ("fa-solid fa-folder cfb-text-[5rem]") }, });
        // @ts-ignore
        [isValid, isValid,];
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({});
        (__VLS_ctx.uploadText);
        // @ts-ignore
        [uploadText,];
    }
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-flex cfb-justify-between cfb-px-2") }, });
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-text-sm cfb-text cfb-text-main-600 cfb-grid") }, });
    if (__VLS_ctx.computedErrorMsg) {
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({ ...{ class: ("cfb-text-danger-700") }, });
        (__VLS_ctx.computedErrorMsg);
        // @ts-ignore
        [computedErrorMsg, computedErrorMsg,];
    }
    if (__VLS_ctx.mimeTypeLimit) {
        __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({});
        (__VLS_ctx.mimeTypeLimitText(__VLS_ctx.mimeTypeLimit));
        // @ts-ignore
        [mimeTypeLimit, mimeTypeLimit, mimeTypeLimitText,];
    }
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({});
    (__VLS_ctx.fileSizeBytesLimitText(__VLS_ctx.fileSizeBytesLimit));
    // @ts-ignore
    [fileSizeBytesLimitText, fileSizeBytesLimit,];
    __VLS_elementAsFunction(__VLS_intrinsicElements.div, __VLS_intrinsicElements.div)({});
    (__VLS_ctx.fileNumberLimitText(__VLS_ctx.fileNumberLimit));
    // @ts-ignore
    [fileNumberLimitText, fileNumberLimit,];
    if (typeof __VLS_styleScopedClasses === 'object' && !Array.isArray(__VLS_styleScopedClasses)) {
        __VLS_styleScopedClasses[''];
        __VLS_styleScopedClasses['cfb-grid'];
        __VLS_styleScopedClasses['cfb-hidden'];
        __VLS_styleScopedClasses['cfb-px-2'];
        __VLS_styleScopedClasses['cfb-py-1'];
        __VLS_styleScopedClasses['cfb-p-4'];
        __VLS_styleScopedClasses['cfb-w-full'];
        __VLS_styleScopedClasses['cfb-grid'];
        __VLS_styleScopedClasses['cfb-grid-cols-1'];
        __VLS_styleScopedClasses['tab:cfb-grid-cols-[repeat(auto-fill,20rem)]'];
        __VLS_styleScopedClasses['cfb-gap-4'];
        __VLS_styleScopedClasses['cfb-place-self-start'];
        __VLS_styleScopedClasses['cfb-flex'];
        __VLS_styleScopedClasses['cfb-gap-4'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['cfb-p-1'];
        __VLS_styleScopedClasses['cfb-w-full'];
        __VLS_styleScopedClasses['cfb-cursor-pointer'];
        __VLS_styleScopedClasses['cfb-w-20'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['cfb-aspect-square'];
        __VLS_styleScopedClasses['cfb-object-cover'];
        __VLS_styleScopedClasses['cfb-self-start'];
        __VLS_styleScopedClasses['cfb-w-20'];
        __VLS_styleScopedClasses['cfb-shrink-0'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['cfb-aspect-square'];
        __VLS_styleScopedClasses['cfb-self-start'];
        __VLS_styleScopedClasses['cfb-bg-gray-100'];
        __VLS_styleScopedClasses['cfb-flex'];
        __VLS_styleScopedClasses['cfb-justify-center'];
        __VLS_styleScopedClasses['cfb-items-center'];
        __VLS_styleScopedClasses['fa-solid'];
        __VLS_styleScopedClasses['fa-file'];
        __VLS_styleScopedClasses['cfb-text-[3rem]'];
        __VLS_styleScopedClasses['cfb-text-main-500'];
        __VLS_styleScopedClasses['cfb-text-xs'];
        __VLS_styleScopedClasses['cfb-flex-shrink'];
        __VLS_styleScopedClasses['cfb-min-w-0'];
        __VLS_styleScopedClasses['cfb-flex-col'];
        __VLS_styleScopedClasses['cfb-flex'];
        __VLS_styleScopedClasses['cfb-flex-grow'];
        __VLS_styleScopedClasses['cfb-text-sm'];
        __VLS_styleScopedClasses['cfb-break-words'];
        __VLS_styleScopedClasses['cfb-flex-grow'];
        __VLS_styleScopedClasses['cfb-mb-2'];
        __VLS_styleScopedClasses['cfb-text-danger-600'];
        __VLS_styleScopedClasses['cfb-self-end'];
        __VLS_styleScopedClasses['cfb-bg-gray-200'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['cfb-h-[4px]'];
        __VLS_styleScopedClasses['cfb-relative'];
        __VLS_styleScopedClasses['cfb-bg-main-500'];
        __VLS_styleScopedClasses['cfb-absolute'];
        __VLS_styleScopedClasses['cfb-h-full'];
        __VLS_styleScopedClasses['cfb-text-danger-600'];
        __VLS_styleScopedClasses['hover:cfb-bg-gray-100'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['cfb-w-5'];
        __VLS_styleScopedClasses['cfb-aspect-square'];
        __VLS_styleScopedClasses['disabled:cfb-text-gray-700'];
        __VLS_styleScopedClasses['fa-solid'];
        __VLS_styleScopedClasses['fa-ban'];
        __VLS_styleScopedClasses['cfb-w-full'];
        __VLS_styleScopedClasses['cfb-justify-center'];
        __VLS_styleScopedClasses['cfb-grid'];
        __VLS_styleScopedClasses['cfb-items-center'];
        __VLS_styleScopedClasses['hover:cfb-bg-main-100'];
        __VLS_styleScopedClasses['cfb-cursor-pointer'];
        __VLS_styleScopedClasses['cfb-rounded-lg'];
        __VLS_styleScopedClasses['fa-solid'];
        __VLS_styleScopedClasses['fa-folder'];
        __VLS_styleScopedClasses['cfb-text-[5rem]'];
        __VLS_styleScopedClasses['cfb-flex'];
        __VLS_styleScopedClasses['cfb-justify-between'];
        __VLS_styleScopedClasses['cfb-px-2'];
        __VLS_styleScopedClasses['cfb-text-sm'];
        __VLS_styleScopedClasses['cfb-text'];
        __VLS_styleScopedClasses['cfb-text-main-600'];
        __VLS_styleScopedClasses['cfb-grid'];
        __VLS_styleScopedClasses['cfb-text-danger-700'];
    }
    var __VLS_slots;
    return __VLS_slots;
    const __VLS_componentsOption = {};
    let __VLS_name;
    let __VLS_defineComponent;
    const __VLS_internalComponent = __VLS_defineComponent({
        setup() {
            return {
                byteSize: byteSize,
                value: value,
                isRequired: isRequired,
                validatedMeta: validatedMeta,
                isValid: isValid,
                uploading: uploading,
                dragging: dragging,
                el: el,
                map: map,
                computedErrorMsg: computedErrorMsg,
                changeHandler: changeHandler,
                dropHandler: dropHandler,
                deleteHandler: deleteHandler,
            };
        },
        props: {},
    });
}
export default (await import('vue')).defineComponent({
    setup() {
        return {
            ...__VLS_exposed,
        };
    },
    props: {},
});
;
