import { deepAssign } from '@codeperate/utils';
import { computed, inject, onBeforeMount, onUnmounted, ref, toRaw } from 'vue';
import { useConfig } from '../config';
export function useFormWidget(name, props) {
    let widget;
    const isValidated = ref(false);
    const validatedMeta = ref({ validity: true, path: props.path });
    const form = inject('formBuilder');
    const isHidden = () => {
        const hidden = props.hidden || schema.value.hidden;
        if (hidden)
            return typeof hidden == 'function' ? hidden(widget) : hidden;
        return false;
    };
    const getSchema = () => form.getSchema(props.path);
    const getValue = () => form.getValue(getPath());
    const getPath = () => props.targetPath ?? getSchema().targetPath ?? props.path;
    const getPathArr = () => getPath()
        .split('.')
        .filter(s => s);
    const setValue = (value) => form.setValue(getPath(), value);
    const isRequired = () => {
        const required = props.required || getSchema().required;
        if (required == null)
            return false;
        return typeof required == 'function' ? required(widget) : required;
    };
    const isValid = () => {
        if (isValidated.value == true)
            return validatedMeta.value.validity;
    };
    const value = computed(() => getValue());
    const schema = computed(() => getSchema());
    const config = computed(() => deepAssign(useConfig(name), schema?.value.config ?? {}));
    const register = _exposed => {
        widget = { ...exposed, ..._exposed };
        onBeforeMount(() => form.regWidget(props.path, widget));
        onUnmounted(() => form.unRegWidget(props.path, widget));
        return widget;
    };
    const validator = () => {
        validatedMeta.value.validity = true;
        validatedMeta.value.err = undefined;
    };
    const view = computed(() => schema.value.view ?? form.getView());
    const validate = () => {
        let result = toRaw(validatedMeta.value);
        const schema = getSchema();
        let { validate } = schema;
        validate = typeof validate == 'function' ? validate(widget) : validate;
        if (!view.value && (validate ?? true)) {
            if (schema.validateFn)
                schema.validateFn(value.value, widget);
            else
                widget.validator();
            isValidated.value = true;
        }
        return result;
    };
    const undoValidate = () => {
        isValidated.value = false;
        validatedMeta.value.validity = false;
    };
    if (getSchema().defaultValue !== undefined && value.value == undefined)
        setValue(getSchema().defaultValue);
    onBeforeMount(() => {
        if (widget == undefined)
            throw new Error('You must call register() function');
    });
    // onRenderTriggered(({ key, target, type }) => {
    //     console.log(name);
    //     console.log(`Render triggered on component`, { key, target, type });
    // });
    const onExportValue = async () => { };
    const onSaveHistory = async () => { };
    const onLoadHistory = async () => { };
    const exposed = {
        isValidated,
        form,
        value,
        validatedMeta,
        view,
        schema,
        config,
        isHidden,
        isValid,
        isRequired,
        getSchema,
        getPath,
        getPathArr,
        setValue,
        getValue,
        register,
        validator,
        validate,
        undoValidate,
        onExportValue,
        onSaveHistory,
        onLoadHistory,
    };
    return exposed;
}
